myShader = false
textureVol = false
textureCube = false
timer = false

local Textures = {}
ShaderText = [[
	#include "mta-helper.fx"
float2 sPixelSize = float2(0,0);
float3 ColorSkyTop;
float3 ColorSkyBottom;
float3 sunPos = float3(0, 0, 0);
float currentWeather = 0;
//---------------------------------------------------------------------
// Textures
//---------------------------------------------------------------------
texture screenInput;
texture normalTexture;
texture foamTexture;
texture AlphaTexture;
texture gDepthBuffer : DEPTHBUFFER;
//---------------------------------------------------------------------
// Sampler
//---------------------------------------------------------------------
sampler2D AlphaSampler = sampler_state
{
   Texture = (AlphaTexture);
   MAGFILTER = LINEAR;
   MINFILTER = LINEAR;
   MIPFILTER = LINEAR;
   AddressU  = Wrap;
   AddressV  = Wrap;
   AddressW = Wrap;
   MIPMAPLODBIAS = 0.0;
};

sampler2D screenSampler = sampler_state
{
	Texture = <screenInput>;
	MinFilter = Linear;
	MagFilter = Linear;
	MipFilter = Linear;
	AddressU = Mirror;
	AddressV = Mirror;
};

sampler2D foamSampler = sampler_state
{
	Texture = <foamTexture>;
	MinFilter = Linear;
	MagFilter = Linear;
	MipFilter = Linear;
	AddressU = Wrap;
	AddressV = Wrap;
};

sampler NormalSampler = sampler_state
{
	Texture = <normalTexture>;
	MinFilter = LINEAR;
	MagFilter = LINEAR;
	MipFilter = LINEAR;
	AddressU  = Wrap;
	AddressV  = Wrap;
	AddressW  = Wrap;	
	SRGBTexture=FALSE;
	MaxMipLevel=0;
	MipMapLodBias=0;
};

sampler SamplerDepth = sampler_state
{
	Texture = (gDepthBuffer);
	MinFilter = Point;
	MagFilter = Point;
	MipFilter = None;
	AddressU = Clamp;
	AddressV = Clamp;
};

//---------------------------------------------------------------------
// Structure of data sent to the vertex shader
//---------------------------------------------------------------------

struct VertexInputType
{
	float4 position : POSITION;
	float3 normal : NORMAL0;
	float2 textureCoords : TEXCOORD0;
};

//---------------------------------------------------------------------
// Structure of data sent to the pixel shader (from the vertex shader)
//---------------------------------------------------------------------

struct PixelInputType
{
	float4 position : POSITION;
	float2 textureCoords : TEXCOORD0;
	float4 worldPosition : TEXCOORD1;
	float3 lightDirection : TEXCOORD2;
	float3 worldNormal : TEXCOORD3;
	float4 vposition : TEXCOORD4;
};
PixelInputType WaterVertexShader(VertexInputType input)
{
	PixelInputType output;

	output.worldPosition = MTACalcWorldPosition(input.position);
	float4 viewPos = mul(output.worldPosition, gView);
	output.worldPosition.w = viewPos.z / viewPos.w;
	output.position = mul(viewPos, gProjection);
	output.lightDirection = normalize(gCameraPosition - sunPos);
	MTAFixUpNormal(input.normal);
	output.worldNormal = MTACalcWorldNormal(input.normal);
	
	// Store the texture coordinates for the pixel shader.
	output.textureCoords = input.textureCoords;

	output.vposition = mul(input.position, gWorldViewProjection);
	return output;
}
float3 WorldScr(float3 position)
{
	float4 pVP = mul(float4(position, 1), gViewProjection);
	pVP.xy = float2(0.5f, 0.5f) + float2(0.5f, -0.5f) * pVP.xy / pVP.w;
	return float3(pVP.xy, pVP.z / pVP.w);
}

float GameTime = 24;

float Luminance(in float3 color)
{
	return dot(color.rgb, float3(0.2125f, 0.7154f, 0.0721f));
}

float3 RayM2(float2 vpos, float3 pos, float3 wpos, float3 normal, float3 sky)
{
   float x1 = smoothstep(0.0, 2.0, GameTime),
         x2a = smoothstep(2.0, 4.0, GameTime),    
         x2 = smoothstep(4.0, 5.0, GameTime),
         x3 = smoothstep(5.0, 6.0, GameTime),
         x4 = smoothstep(6.0, 7.0, GameTime),
         x5e = smoothstep(8.0, 11.0, GameTime),
         x5 = smoothstep(16.0, 17.0, GameTime),
         x6 = smoothstep(18.0, 19.0, GameTime),
         x7 = smoothstep(19.0, 20.0, GameTime),
         x8 = smoothstep(22.0, 23.0, GameTime);	
	
	float3 view = normalize(wpos.xyz - gCameraPosition.xyz);
	float l = length(wpos.xyz - gCameraPosition.xyz)/0.1;	
	float l2 = length(view.xyz);		
    float3 camdir = view.xyz / l2;	
	float3 refl = reflect(view*float3(1.0, 1.0, -1.0), normal*0.55);
		   refl = refl/1.0;

	float3 p = wpos+(refl*l);
			
    float2 uv = WorldScr(p);
	  
    float a1 = saturate(6.7*(uv.y)),
          a2 = saturate(6.7+uv.y*(-6.7));	

    float fa = 0.01-dot(normal, -camdir);		
		  fa = saturate(pow(fa, 4.0));
		   
	float4 ssr = 1.0;	
	       ssr.rgb = tex2Dlod(screenSampler, float4(uv, 0.0, 0.0)).rgb;
	       ssr.w = uv.y<0.0||uv.y>1.0 ? 0.0:1.0; 	   
	
    float nf1 = saturate(10.0*(uv.y)),	 
          nf2 = saturate(10.0+uv.y*(-10.0));	

    float sky2 = sky;
    float3 sky3 = lerp(sky2, sky, 0.2);
		   
    float wx = currentWeather;
    float3 skyA = sky3*0.5;

if (wx.x==0) skyA*= 2.0;
if (wx.x==1) skyA*= 1.5;
		   
    float3 ti = lerp(0.25, 0.25, x1);
           ti = lerp(ti, 0.2, x2a);    
           ti = lerp(ti, 0.3, x2);     
           ti = lerp(ti, skyA, x3);   
           ti = lerp(ti, skyA, x4); 		  
           ti = lerp(ti, skyA, x5e);
           ti = lerp(ti, skyA, x5);
           ti = lerp(ti, skyA, x6);	  
           ti = lerp(ti, skyA*0.8, x7);	
           ti = lerp(ti, 0.25, x8);
		   
     float ti2 = lerp(0.6, 0.6, x1);
           ti2 = lerp(ti2, 0.6, x2a);    
           ti2 = lerp(ti2, 0.8, x2);     
           ti2 = lerp(ti2, 0.7, x3);   
           ti2 = lerp(ti2, 0.6, x4); 		  
           ti2 = lerp(ti2, 0.5, x5e);
           ti2 = lerp(ti2, 0.5, x5);
           ti2 = lerp(ti2, 0.5, x6);	  
           ti2 = lerp(ti2, 0.8, x7);	
           ti2 = lerp(ti2, 0.7, x8);		   
	
	sky*=ti2;
	
    float3 reflection = lerp(sky, ssr.rgb*ti*1.2, ssr.w);
		   reflection = lerp(sky, reflection, nf1*nf2);				   
	return reflection;
}

float3 tonemapping(float3 color)
{
//=Uncharted2Tonemap=
    float A = 1.0;
	float B = 0.25;
	float C = 0.52;
	float D = 0.34;
	float E = 0.0;
	float F = 1.0;

    float3 tm1 = ((color*(A*color+C*B)+D*E)/(color*(A*color+B)+D*F))-E/F;
//===================
	float3 n0 = normalize(color.xyz);
	float3 ct0=color.xyz/n0.xyz;
	       ct0=pow(ct0, 1.0);
           n0.xyz = pow(n0.xyz, 0.9);   
           color.xyz = ct0*n0.xyz;  
           color.xyz*= 0.9;  
		   
    float maxcol = max(color.x, max(color.y, color.z));		 		 
    float3 r0 = 1.0-exp(-maxcol),
           r2 = 1.0-exp(-color);	

         float3 tm0 = lerp(float3(r2.x, r2.y, r2.z), color*float3(r0.x, r0.y, r0.z)/maxcol, 0.2);		 
		 
	     color.rgb = lerp(tm0, tm1, 0.25);	 
  return color;	
}

float4 WaterPixelShader(PixelInputType input, float2 vPos : VPOS0) : COLOR0
{
	float4 finalColor = 1.0;
	float t0 = GameTime;
	float x1 = smoothstep(0.0, 4.0, t0);
	float x2 = smoothstep(4.0, 5.0, t0);
	float x3 = smoothstep(5.0, 6.0, t0);
	float x4 = smoothstep(6.0, 7.0, t0);
	float x5e = smoothstep(8.0, 11.0, t0);
	float x5 = smoothstep(16.0, 17.0, t0);
	float x6 = smoothstep(18.0, 19.0, t0);
	float x7 = smoothstep(19.0, 20.0, t0);
	float x8 = smoothstep(22.0, 24.0, t0);

	float2 txcoord = (input.vposition.xy / input.vposition.w) * float2(0.5, -0.5) + 0.5;
	       txcoord += 0.5 * sPixelSize;

    float d0 = tex2D(SamplerDepth, txcoord.xy).x; 
	float d1 = input.vposition.z/input.vposition.w;
	      d1 = 1.0/(1-d1);

	float3 v = normalize(input.worldPosition - gCameraPosition);
	float3 v2 = input.worldPosition.xyz - gCameraPosition.xyz;
	
	float4 orig;	
	float4 col;	
	float2 of = 0.0;
    float3 normal;
	float2 coord;
	float2 uv1, uv2, uv3, uv4, uv5;
	
    float time = gTime*0.02;
	float ts = 0.07;	
	float ts2 = 0.05;

	uv1.xy = float2(0.38,0.45)*ts2*input.worldPosition.xy+float2(ts*10.00, -ts*5.70)*time;
	uv2.xy = float2(1.80,2.00)*ts2*input.worldPosition.xy+float2(ts*11.00, -ts*7.75)*time;
	uv3.xy = float2(10.0,10.2)*ts2*input.worldPosition.xy+float2(-ts*5.78,  ts*7.72)*time;
	uv4.xy = float2(0.13,0.20)*ts2*input.worldPosition.xy+float2(ts*8.40, -ts*10.10)*time;
	uv5.xy = float2(1.10,1.30)*ts2*input.worldPosition.xy+float2(-ts*5.40, ts*15.10)*time;	
	
	col = tex2D(NormalSampler, uv1);
	col+= tex2D(NormalSampler, uv2);
	col+= tex2D(NormalSampler, uv3);	
	col+= tex2D(NormalSampler, uv4);
	col+= tex2D(NormalSampler, uv5);		
	orig = col*0.25;
	
    float4 fx = tex2D(AlphaSampler, 0.07*0.0010 + uv4);	
	float2 uvshift = -v2.xy * (orig.x*2.0-1.0) * 0.001*0.3;
	uv1.xy+= uvshift;
	uv2.xy+= uvshift;
	uv3.xy+= uvshift;	
	uv4.xy+= uvshift;
	uv5.xy+= uvshift;	
	col = tex2D(NormalSampler, uv1);
	col+= tex2D(NormalSampler, uv2);
	col+= tex2D(NormalSampler, uv3);	
	col+= tex2D(NormalSampler, uv4);
	col+= tex2D(NormalSampler, uv5);	
	orig = col*0.25;		
	
	float2 kernel[4]=
	{
		float2( 1.0, 0.0),
		float2(-1.0, 0.0),
		float2( 0.0, 1.0),
		float2( 0.0,-1.0)
	};
	for (int i=0; i<4; i++)
	{
		coord = kernel[i];
		float4 tc;
		float4 tc4;
		float4 tc5;	
		
		tc = tex2D(NormalSampler, 0.07*coord*0.0010 + uv1.xy);
		tc4 = tex2D(NormalSampler, 0.07*coord*0.0010 + uv4.xy);
		tc5 = tex2D(NormalSampler, 0.07*coord*0.0010 + uv5.xy);		

		tc.x+= tc4.x;	
		tc.x+= tc5.x;			
		of+= coord*(col.x-tc.x);				
	}

	float EWaveBumpiness = 0.65;
	       normal.xy = of.xy*10.0*2.20*EWaveBumpiness;
	       normal.z = sqrt(1.0-dot(of.xy, of.xy));	
	       normal = normalize(mul(normal, (float3x3)gWorld));			   
		   
    float3 refl = normalize(reflect(v, normal));
		
    float3 vec1 = normalize(float3(0.0, 0.0, 1.0)); 
    float sky = saturate(1.0 - dot(vec1, refl));
	      sky = pow(sky, 3.0);

   float3 SkyColor = lerp(gFogColor.rgb*1.5,ColorSkyTop*0.001, 1-sky);
	
		 //finalColor.xyz = RayM(float2(1,1), input.worldPosition.xyz, input.worldPosition.xyz, normal.xyz, SkyColor*0.8);	
		 finalColor.xyz = RayM2(float2(1,1), input.worldPosition.xyz, input.worldPosition.xyz, normal.xyz, SkyColor*0.8);	
		 
	float depthF2 = saturate(pow(1.0/(1-d0) - d1, 1.5)*1.0);
	float depthF1 = pow(1.0/(1-d0) - d1, 0.45)*0.15+depthF2*0.3;				  
		  
    float wave1 = tex2D(foamSampler, uv2.xy*2.0),
          wave2 = tex2D(foamSampler, uv5.xy*1.0);			  
 
		 //finalColor.a = depthF1;
		   
    float t2 = lerp(0.2, 0.2, x1);
          t2 = lerp(t2, 0.5, x2);
          t2 = lerp(t2, 1.0, x3);
          t2 = lerp(t2, 1.0, x4);
          t2 = lerp(t2, 1.0, x5e);
          t2 = lerp(t2, 1.0, x5);
          t2 = lerp(t2, 1.0, x6);	  	 
          t2 = lerp(t2, 0.8, x7);			   
          t2 = lerp(t2, 0.2, x8);  	  

	       fx.xyz+= 0.000001;	
	float3 st4 = normalize(fx.xyz);
	float3 ct4 = fx.xyz/st4.xyz;
	       ct4 = pow(ct4, 5.50);
	       st4.xyz = pow(st4.xyz, 3.93);
	       fx.xyz = ct4*st4.xyz;
           fx.xyz*= 0.23;
    float af0 = saturate(fx);
	finalColor.rgb = lerp(finalColor.rgb, lerp(finalColor.rgb, t2*2.2, wave2*wave1), af0); 

 //SunDirection--------------------
float3  f = lerp(float3(0.,0.,-.61), float3(.583,-.257,-.0108), smoothstep(0.,5.,GameTime));
        f = lerp(f, float3(.569,-.242,.151), smoothstep(5.,7.,GameTime));
		f = lerp(f, float3(.49,-.247,.323), smoothstep(7.,9.,GameTime));
		f = lerp(f, float3(.373,-.262,.445), smoothstep(9.,11.,GameTime));
		f = lerp(f, float3(.201,-.312,.517), smoothstep(11.,13.,GameTime));
		f = lerp(f, float3(-.00732,-.39,.503), smoothstep(13.,15.,GameTime));
		f = lerp(f, float3(-.189,-.502,.343), smoothstep(15.,17.,GameTime));
		f = lerp(f, float3(-.223,-.594,.1), smoothstep(17.,19.,GameTime));
		f = lerp(f, float3(0.,0.,-.61), smoothstep(19.,24.,GameTime));
		f = normalize(f); 
	float3 camdir = gCameraPosition.xyz - input.worldPosition.xyz;	
	       camdir = normalize(camdir);
	
	float3 vec = normalize(camdir+f*float3(1.02, 1.02, 1.2));	
		  
    float specular = 0.001-dot(-vec, normal.xyz);
          specular = pow(specular, 100.0*0.5);	
          specular/= 2.2;
  
    float3 t1 = lerp(0.0, 0.0, x1);
           t1 = lerp(t1, 0.0, x2);
           t1 = lerp(t1, float3(0.431, 0.375, 0.296)*0.7, x3);
           t1 = lerp(t1, float3(0.431, 0.375, 0.296)*0.8, x4);
           t1 = lerp(t1, float3(0.216, 0.216, 0.216)*2.0, x5e);
           t1 = lerp(t1, float3(0.216, 0.216, 0.216)*2.0, x5);
           t1 = lerp(t1, float3(0.431, 0.375, 0.296)*0.8, x6);	  	 
           t1 = lerp(t1, 0.0, x7);			   
           t1 = lerp(t1, 0.0, x8);  

    float wx = currentWeather;
    float sunA = 2.0;

    if (wx==4) sunA = 0.0;
    if (wx==7) sunA = 0.0;
    if (wx==8) sunA = 0.0;
    if (wx==9) sunA = 0.0;
    if (wx==12) sunA = 0.0;
    if (wx==15) sunA = 0.0;
    if (wx==16) sunA = 0.0;	  	   

		 finalColor.rgb+= specular*t1*sunA;
		 finalColor.rgb = tonemapping(finalColor.rgb*1.0);
		 
    float pos = frac(vPos.xy/2.0) + 0.25;
    float3 j0 = float3(0.25,-0.25,0.25) * (1.0 / (exp2(8.0) - 1.0));
     	   j0 = lerp(2.0 * j0, -2.0 * j0, pos);	
		   
		 finalColor.rgb+= j0;		   
		 
  return float4(MTAApplyFog(finalColor.rgb, input.worldPosition), finalColor.a);
}

technique water
{
    pass P0
    {
        VertexShader = compile vs_3_0 WaterVertexShader();
        PixelShader  = compile ps_3_0 WaterPixelShader();
    }
}


technique fallback
{
    pass P0
    {
    }
}

]]

local myShader
local width, height = guiGetScreenSize()

function start()
	myShader = dxCreateShader(ShaderText, 999, 0, false, "world")

	if myShader then
		Textures.waterNormal = dxCreateTexture("images/normal.png", "dxt5")
		Textures.waterFoam = dxCreateTexture("images/foam.png", "dxt5")
		Textures.waterA = dxCreateTexture("images/WaterAlpha.png", "dxt5")
		-- setNearClipDistance(1.0)	
		dxSetShaderValue(myShader, "sPixelSize", {1/width, 1/height})
		dxSetShaderValue(myShader, "normalTexture", Textures.waterNormal)
		dxSetShaderValue(myShader, "foamTexture", Textures.waterFoam)
		dxSetShaderValue(myShader, "AlphaTexture", Textures.waterA)		
		engineApplyShaderToWorldTexture ( myShader, "waterclear256" )
		addEventHandler("onClientRender", root, onRender)
		isRender = true
	end
end

function onRender()
	if not isElement(screen) then
		local resName = "paint" -- тяну с другого ресурса, чтобы не копировать одинаковые текстурки, да и обновление только в одном месте текстуры
		if getResourceFromName(resName) and getResourceState(getResourceFromName(resName)) == "running" then
			screen = exports[resName]:getScreenSource()
		else
			return
		end
	end
	-- dxUpdateScreenSource(screen)								
	local tempHour, minutes = getTime()
	minutes = (1/ 59) * minutes	
	dxSetShaderValue(myShader, "GameTime", tempHour+minutes)	
	dxSetShaderValue(myShader, "screenInput", screen)	
	dxSetShaderValue(myShader, "currentWeather", getWeather())
	local r,g,b,r1,g1,b1 = getSkyGradient()			
	dxSetShaderValue(myShader, "ColorSkyTop", { r,g,b })
	dxSetShaderValue(myShader, "ColorSkyBottom", { r1,g1,b1 })	
end

function stop()
	resetNearClipDistance()
	if isRender then
		removeEventHandler("onClientRender", root, onRender)
		isRender = false
	end	
	if isElement(myShader) then engineRemoveShaderFromWorldTexture(myShader, "*") myShader:destroy() end
	for _, texture in pairs(Textures) do
		if isElement(texture) then destroyElement(texture) end
	end
end